@@ -52,7 +52,7 @@ function bytes.is_all(bs,byte)
 	return true
 end
 
-
+--[[
 LFI_LIST = {
   { "ICC",              "/0002",      "file" },
   { "ID",               "/0003",      "file" },
@@ -75,6 +75,69 @@ LFI_LIST = {
   { "Counters",         "/2000/2069", "file" },
   { "Holder Extended",  "/3F1C",      "file" }
 }
+--]]
+LFI_LIST = {
+
+  { "AID",              "/3F04",      "file" },
+  { "ICC",              "/0002",      "file" },
+  { "ID",               "/0003",      "file" },
+  { "Holder Extended",  "/3F1C",      "file" },
+  { "Display / Free",   "/2F10",      "file" },
+  
+  { "Ticketing",        "/2000",    "folder" },
+  { "AID",              "/2000/2004", "file" },
+  { "Environment",      "/2000/2001", "file" },
+  { "Holder",           "/2000/2002", "file" }, 
+  { "Event logs",       "/2000/2010", "file" },
+  { "Contracts",        "/2000/2020", "file" },
+  { "Contracts",        "/2000/2030", "file" },
+  { "Counters",         "/2000/202A", "file" },
+  { "Counters",         "/2000/202B", "file" },
+  { "Counters",         "/2000/202C", "file" },
+  { "Counters",         "/2000/202D", "file" },
+  { "Counters",         "/2000/202E", "file" },
+  { "Counters",         "/2000/202F", "file" },
+  { "Counters",         "/2000/2060", "file" },
+  { "Counters",         "/2000/2061", "file" },
+  { "Counters",         "/2000/2062", "file" },
+  { "Counters",         "/2000/2069", "file" },
+  { "Counters",         "/2000/206A", "file" },
+  { "Special events",   "/2000/2040", "file" },
+  { "Contract list",    "/2000/2050", "file" },
+  { "Free",             "/2000/20F0", "file" },
+ 
+  { "MPP",              "/3100",    "folder" },
+    { "AID",            "/3100/3104", "file" },
+    { "Public Param.",  "/3100/3102", "file" },
+    { "Log",            "/3100/3115", "file" },
+    { "Contracts",      "/3100/3120", "file" },
+    { "Counters",       "/3100/3113", "file" },
+    { "Counters",       "/3100/3123", "file" },
+    { "Counters",       "/3100/3133", "file" },
+    { "Counters",       "/3100/3169", "file" },
+    { "Miscellaneous",  "/3100/3150", "file" },
+    { "Free",           "/3100/31F0", "file" },
+        
+  { "RT2",              "/2100",    "folder" },
+    { "AID",            "/2100/2104", "file" },
+    { "Environment",    "/2100/2101", "file" },
+    { "Event logs",     "/2100/2110", "file" },
+    { "Contract list",  "/2100/2150", "file" },
+    { "Contracts",      "/2100/2120", "file" },
+    { "Counters",       "/2100/2169", "file" },
+    { "Special events", "/2100/2140", "file" },
+    { "Free",           "/2100/21F0", "file" },
+      
+  { "EP",               "/1000",    "folder" },
+   { "AID",             "/1000/1004", "file" },
+   { "Load Log",        "/1000/1014", "file" },
+   { "Purchase Log",    "/1000/1015", "file" },
+   
+  { "eTicket",          "/8000",    "folder" },
+   { "AID",             "/8000/8004", "file" },
+   { "Preselection",    "/8000/8030", "file" },
+   { "Event logs",      "/8000/8010", "file" }
+}
 
 function calypso_select(ctx,desc,path,klass)
 	local path_parsed = card.make_file_path(path)
@@ -129,7 +192,7 @@ function calypso_guess_network(cardenv)
 				local country_num = tonumber(country_bin:convert(4):format("%D"))
 			    local network_num = tonumber(network_bin:convert(4):format("%D"))
 
-				if country_num==250 or country_num==56 then
+				if country_num==250 or country_num==56 or country_num==131 then
 					return country_num, network_num
 				end
 				
