@@ -1,8 +1,8 @@
 /********************************************************************** 
 *
-* This file is part of Cardpeek, the smartcard reader utility.
+* This file is part of Cardpeek, the smart card reader utility.
 *
-* Copyright 2009-2010 by 'L1L1'
+* Copyright 2009-2013 by Alain Pannetrat <L1L1@gmx.com>
 *
 * Cardpeek is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,10 @@
 
 int luaopen_bytes(lua_State* L);
 
-bytestring_t* luaL_checkbytestring(lua_State *L, int p);
+bytestring_t* luaL_check_bytestring(lua_State *L, int p);
 
-void lua_pushbytestring(lua_State *L, bytestring_t* bs);
+int luaL_is_bytestring(lua_State *L, int p);
+
+void lua_push_bytestring(lua_State *L, bytestring_t* bs);
 
 #endif
