@@ -20,48 +20,60 @@
 */
 #include "system_info.h"
 
+/* this hash function is used by SDBM. */
+static unsigned int simple_hash(const char *str)
+{
+    unsigned int hash = 0;
+    int c;
+
+    while ((c = *str++)!=0)
+        hash = c + (hash << 6) + (hash << 16) - hash;
+    return hash;
+}
+
 #ifndef _WIN32
 #include <string.h>
 #include <stdio.h>
 #include "config.h"
+#include <sys/utsname.h>
+
+const char *system_type(void)
+{
+    static char info[32];
+    static struct utsname u;
+    
+    if (uname(&u)<0)    
+    {       
+        return "Unknown";      
+    }
+    snprintf(info,32,"%s",u.sysname);
+    
+    return info;
+}
 
 const char *system_string_info(void)
 {
-	static char info[128];
-	char first_line[128];
-	FILE *cmd;
-	char *p;
-
-	if ((cmd=popen("uname -smr","r")))
-	{
-		if (fgets(first_line,127,cmd))
-		{
-			for (p=first_line;*p!='\0';p++)
-				if (*p=='\r' || *p=='\n') 
-				{ 
-					*p='\0'; 
-					break; 
-				} 	
-		}
-		else 
-		{
-			strcpy(first_line,"Unknown os");
-		}
-		if (pclose(cmd),0)
-		{
-			strcpy(first_line,"Unknown os");
-		}
-	}
-	else
-	{
-		strcpy(first_line,"Unknown os");
-	}
-	snprintf(info,128,"Cardpeek %s on %s", 
-		VERSION,
-		first_line);
-	return info;	
+    static char info[128];
+    static struct utsname u;
+    
+    if (uname(&u)<0)    
+    {       
+        return "Unknown OS";      
+    }
+    snprintf(info,128,"%s %s %s",u.sysname,u.release,u.machine);
+    
+    return info;
 }
 
+unsigned int system_name_hash(void)
+{
+    static struct utsname u;
+    if (uname(&u)<0)
+    {
+        return 0;
+    }
+    return simple_hash(u.nodename);
+}
 
 #else
 
@@ -70,6 +82,11 @@ const char *system_string_info(void)
 #include <stdio.h>
 #include "win32/config.h"
 
+const char *system_type(void)
+{
+	return "Windows";
+}
+
 typedef void (WINAPI *PGNSI)(LPSYSTEM_INFO);
 
 const char *system_string_info(void)
@@ -112,5 +129,14 @@ const char *system_string_info(void)
 	return info;
 }
 
+unsigned int system_name_hash() 
+{
+    static char computer_name[1024]; 
+    DWORD size = 1024;     
+    GetComputerName(computer_name,&size);           
+    return simple_hash(computer_name);   
+}
+
+
 #endif
 
