@@ -2,7 +2,7 @@
 *
 * This file is part of Cardpeek, the smart card reader utility.
 *
-* Copyright 2009-2013 by Alain Pannetrat <L1L1@gmx.com>
+* Copyright 2009-2014 by Alain Pannetrat <L1L1@gmx.com>
 *
 * Cardpeek is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
 */
 #ifndef SYSTEM_INFO_H
 
+const char *system_type(void);
+
 const char *system_string_info(void);
 
+unsigned int system_name_hash(void);
+
 #endif
